@@ -56,8 +56,9 @@ connect_now_save_settings(ConnectNowHomePlugin* desktop_plugin)
   filename = g_strconcat (g_get_home_dir(), CONNECT_NOW_HOME_PLUGIN_SETTINGS_FILE, NULL);
   g_key_file_load_from_file (keyFile, filename, G_KEY_FILE_KEEP_COMMENTS, NULL);
 
-  g_key_file_set_string (keyFile, desktop_plugin->iD, "connection_id", desktop_plugin->connection_id);
-  g_key_file_set_string (keyFile, desktop_plugin->iD, "connection_name", desktop_plugin->connection_name);
+  g_key_file_set_string(keyFile, desktop_plugin->iD, "connection_id", desktop_plugin->connection_id);
+  g_key_file_set_string(keyFile, desktop_plugin->iD, "connection_name", desktop_plugin->connection_name);
+  g_key_file_set_integer(keyFile, desktop_plugin->iD, "widget_size", desktop_plugin->widget_size);
 
   fileData = g_key_file_to_data (keyFile, &size, NULL);
   g_file_set_contents(filename, fileData, size, NULL);
@@ -67,6 +68,13 @@ connect_now_save_settings(ConnectNowHomePlugin* desktop_plugin)
   g_free (filename); 
 }
 
+static void
+connect_now_settings_connection_changed(HildonPickerButton* button,
+					gpointer data)
+{
+  gtk_entry_set_text(GTK_ENTRY(data), hildon_button_get_value(HILDON_BUTTON(button)));
+}
+
 static void
 connect_now_show_settings_dialog(GtkWidget* widget, gpointer data)
 {
@@ -79,11 +87,21 @@ connect_now_show_settings_dialog(GtkWidget* widget, gpointer data)
   GtkWidget* selector;
   GSList* iap_list;
   GSList* iap_entry;
+  GtkWidget* sizeScale = NULL;
   HildonTouchSelectorColumn* column = NULL;
 
-  dialog = hildon_picker_dialog_new(NULL);
+  dialog = gtk_dialog_new_with_buttons("Settings",
+				       NULL,
+				       0,
+				       "Save",
+				       GTK_RESPONSE_ACCEPT,
+				       NULL);
+  
   selector = hildon_touch_selector_new();
   connection_list = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_STRING);
+  sizeScale = gtk_hscale_new_with_range(50, 200, 10);
+  gtk_scale_set_digits(GTK_SCALE(sizeScale), 0);
+  gtk_range_set_value(GTK_RANGE(sizeScale), desktop_plugin->widget_size);
 
   iap_list = con_ic_connection_get_all_iaps(desktop_plugin->con);
 
@@ -109,24 +127,59 @@ connect_now_show_settings_dialog(GtkWidget* widget, gpointer data)
   hildon_touch_selector_set_column_selection_mode (HILDON_TOUCH_SELECTOR (selector),
 						   HILDON_TOUCH_SELECTOR_SELECTION_MODE_SINGLE);
 
-  hildon_picker_dialog_set_selector(HILDON_PICKER_DIALOG(dialog), HILDON_TOUCH_SELECTOR(selector));
+  GtkWidget* connectionButton = hildon_picker_button_new(HILDON_SIZE_THUMB_HEIGHT, HILDON_BUTTON_ARRANGEMENT_VERTICAL);
+  hildon_picker_button_set_selector(HILDON_PICKER_BUTTON(connectionButton),
+				    HILDON_TOUCH_SELECTOR(selector));
 
+  GtkWidget* content = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
+  GtkWidget* displayNameBox = gtk_hbox_new(TRUE, 3);
+  GtkWidget* displayNameLabel = gtk_label_new("Displayname:");
+  GtkWidget* displayNameEntry = hildon_entry_new(HILDON_SIZE_FINGER_HEIGHT | HILDON_SIZE_AUTO_WIDTH);
+  gtk_entry_set_text(GTK_ENTRY(displayNameEntry), desktop_plugin->connection_name);
+
+  g_signal_connect(G_OBJECT(connectionButton),"value-changed",
+		   G_CALLBACK(connect_now_settings_connection_changed),
+		   displayNameEntry);
+
+  gtk_box_pack_start(GTK_BOX(displayNameBox), displayNameLabel, TRUE, TRUE, 3);
+  gtk_box_pack_end(GTK_BOX(displayNameBox), displayNameEntry, TRUE, TRUE, 3);
+
+  GtkWidget* connectionNameBox = gtk_hbox_new(TRUE, 3);
+  GtkWidget* connectionNameLabel = gtk_label_new("Connection:");
+
+  gtk_box_pack_start(GTK_BOX(connectionNameBox), connectionNameLabel, TRUE, TRUE, 3);
+  gtk_box_pack_end(GTK_BOX(connectionNameBox), connectionButton, TRUE, TRUE, 3);
+
+  gtk_box_pack_start(GTK_BOX(content), displayNameBox, FALSE, FALSE, 3);
+  gtk_box_pack_start(GTK_BOX(content), connectionNameBox, FALSE, FALSE, 3);
+  gtk_box_pack_end(GTK_BOX(content), sizeScale, FALSE, FALSE, 3);
   gtk_widget_show_all(dialog);
 
-  if(gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK)
+  if(gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT)
   {
     GtkTreeIter iter;
-    gchar *name;
     gchar *id;
     if (hildon_touch_selector_get_selected(HILDON_TOUCH_SELECTOR(selector), 0, &iter))
     {
-      gtk_tree_model_get(GTK_TREE_MODEL(connection_list), &iter, 0, &name, 1, &id,  -1);
+      const gchar* displayName = hildon_entry_get_text(HILDON_ENTRY(displayNameEntry));
+      gtk_tree_model_get(GTK_TREE_MODEL(connection_list), &iter, 1,  &id,  -1);
 
       g_free(desktop_plugin->connection_name);
       g_free(desktop_plugin->connection_id);
       desktop_plugin->connection_id = id;
-      desktop_plugin->connection_name = name;
-
+      
+      if(displayName!=NULL && displayName[0]!='\0')
+      {
+	desktop_plugin->connection_name = g_strdup(displayName);
+      }
+      else
+      {
+	gtk_tree_model_get(GTK_TREE_MODEL(connection_list), &iter, 0, &(desktop_plugin->connection_name),  -1);
+      }
+      desktop_plugin->widget_size = gtk_range_get_value(GTK_RANGE(sizeScale));
+      gtk_window_resize(GTK_WINDOW(desktop_plugin),
+			(int)(desktop_plugin->widget_size),
+			(int)(desktop_plugin->widget_size));
       if(g_strcmp0(desktop_plugin->current_connection_id,
 		   desktop_plugin->connection_id)==0)
       {
@@ -166,6 +219,10 @@ read_settings(ConnectNowHomePlugin *desktop_plugin)
 									   FALSE,
 									   NULL);
       }
+    if(g_key_file_has_key(keyFile, desktop_plugin->iD, "widget_size", NULL))
+      {
+	desktop_plugin->widget_size = g_key_file_get_integer(keyFile, desktop_plugin->iD, "widget_size", NULL);
+      }
     if(g_key_file_has_key(keyFile, desktop_plugin->iD, "disconnectedIcon", NULL))
       {
 	desktop_plugin->disconnectedImageName = g_key_file_get_string(keyFile, desktop_plugin->iD, "disconnectedIcon", NULL);
@@ -358,14 +415,14 @@ connect_now_home_plugin_init(ConnectNowHomePlugin *desktop_plugin)
   desktop_plugin->this_connection_state = CON_IC_STATUS_DISCONNECTED;
   desktop_plugin->connect_now = FALSE;
   desktop_plugin->con = con_ic_connection_new();
-
+  desktop_plugin->widget_size = CONNECT_NOW_WIDGET_WIDTH;
 
   g_signal_connect(desktop_plugin->con, "connection-event", G_CALLBACK(connect_now_connection_changed), desktop_plugin);
   g_object_set(desktop_plugin->con, "automatic-connection-events", TRUE);
 
   GtkWidget *contents = build_ui();
 
-  gtk_widget_set_size_request(contents, CONNECT_NOW_WIDGET_WIDTH, CONNECT_NOW_WIDGET_WIDTH);
+  gtk_widget_set_size_request(contents, desktop_plugin->widget_size, desktop_plugin->widget_size);
   gtk_container_add(GTK_CONTAINER(desktop_plugin), contents);
   hd_home_plugin_item_set_settings (HD_HOME_PLUGIN_ITEM (desktop_plugin), TRUE);
   g_signal_connect(GTK_CONTAINER(contents), "button-press-event", G_CALLBACK(connect_now_button_press), desktop_plugin);
@@ -381,6 +438,10 @@ connect_now_realize(GtkWidget* widget)
   gtk_widget_set_app_paintable(widget, TRUE);
   CONNECT_NOW_HOME_PLUGIN(widget)->iD = hd_home_plugin_item_get_applet_id (HD_HOME_PLUGIN_ITEM (widget));
   read_settings(CONNECT_NOW_HOME_PLUGIN(widget));
+  gtk_window_resize(GTK_WINDOW(widget),
+		    CONNECT_NOW_HOME_PLUGIN(widget)->widget_size,
+		    CONNECT_NOW_HOME_PLUGIN(widget)->widget_size);
+
   GTK_WIDGET_CLASS(connect_now_home_plugin_parent_class)->realize(widget);
 }
 
@@ -395,6 +456,8 @@ connect_now_expose(GtkWidget* widget, GdkEventExpose *event)
   gdk_cairo_region(cr, event->region);
   cairo_clip(cr);
   cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
+  int width = CONNECT_NOW_HOME_PLUGIN(widget)->widget_size;
+  cairo_scale(cr, width/100.0, width/100.0);
 
   if(CONNECT_NOW_HOME_PLUGIN(widget)->connectedImage &&
      CONNECT_NOW_HOME_PLUGIN(widget)->disconnectedImage)
